# Onsite energies database, loaded from GAPW lda potentials. stored as 
# A dictionary of dictionaries. The first dictionary is the element name, and the
# second dictionary is the orbital name. The orbital name is the key, and the value is the onsite  energy.


#
# Contains the elements as follows:

#    AtomSymbol=[
#     'H',                                                                                                  'He', 
#     'Li', 'Be',                                                             'B',  'C',  'N',  'O',  'F',  'Ne', 
#     'Na', 'Mg',                                                             'Al', 'Si', 'P',  'S',  'Cl', 'Ar',
#     'K',  'Ca', 'Sc', 'Ti', 'V',  'Cr', 'Mn', 'Fe', 'Co', 'Ni', 'Cu', 'Zn', 'Ga', 'Ge', 'As', 'Se', 'Br', 'Kr', 
#     'Rb', 'Sr', 'Y',  'Zr', 'Nb', 'Mo',     , 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', 'In', 'Sn', 'Sb', 'Te', 'I',  'Xe', 
#     'Cs', 'Ba',       'Hf', 'Ta', 'W',  'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', 'Tl', 'Pb', 'Bi',             'Rn'
#     ]

onsite_energy_database = \
{
    "H": {
        "1s": -0.23348,
        "s*": 0.76652,
        "p*": 0.0
    },
    "He": {
        "1s": -0.57022,
        "s*": 0.42978,
        "p*": 0.0
    },
    "Li": {
        "2s": -0.10561,
        "2p": -0.04138,
        "s*": 0.89439
    },
    "Be": {
        "2s": -0.20578,
        "2p": -0.07713,
        "s*": 0.79422
    },
    "B": {
        "2s": -0.34482,
        "2p": -0.13648,
        "s*": 0.65518,
        "p*": 0.86352,
        "d*": 0.0
    },
    "C": {
        "2s": -0.50121,
        "2p": -0.19897,
        "s*": 0.49879,
        "p*": 0.80103,
        "d*": 0.0
    },
    "N": {
        "2s": -0.6769242006071096,
        "2p": -0.2659669180262646,
        "s*": 0.32307579939289044,
        "p*": 0.7340330819737354,
        "d*": 0.0
    },
    "O": {
        "2s": -0.8728659519510436,
        "2p": -0.3379245389499657,
        "s*": 0.12713404804895645,
        "p*": 0.6620754610500343,
        "d*": 0.0
    },
    "F": {
        "2s": -1.08949,
        "2p": -0.41501,
        "s*": -0.08949,
        "p*": 0.58499,
        "d*": 0.0
    },
    "Ne": {
        "2s": -1.32708,
        "2p": -0.49727,
        "s*": -0.32708,
        "p*": 0.50273,
        "d*": 0.0
    },
    "Na": {
        "3s": -0.10360710201379364,
        "3p": -0.028428857388198774,
        "s*": 0.8963928979862064,
        "2p": -1.0593253390075597,
        "d*": 0.0
    },
    "Mg": {
        "2s": -2.913889930466784,
        "3s": -0.17572,
        "2p": -1.7171029862790355,
        "3p": -0.05048,
        "d*": 0.0,
        "s*": 0.82428
    },
    "Al": {
        "3s": -0.28773,
        "3p": -0.10229,
        "s*": 0.71227,
        "p*": 0.89771,
        "d*": 0.0
    },
    "Si": {
        "3s": -0.39975,
        "3p": -0.15295,
        "s*": 0.60025,
        "p*": 0.84705,
        "d*": 0.0
    },
    "P": {
        "3s": -0.51503,
        "3p": -0.20565,
        "s*": 0.48497,
        "p*": 0.79435,
        "d*": 0.0
    },
    "S": {
        "3s": -0.6349585962744148,
        "3p": -0.26113899178863764,
        "s*": 0.36504140372558525,
        "p*": 0.7388610082113624,
        "d*": 0.0
    },
    "Cl": {
        "3s": -0.76028,
        "3p": -0.31974,
        "s*": 0.23972,
        "p*": 0.68026,
        "d*": 0.0
    },
    "Ar": {
        "3s": -0.89144,
        "3p": -0.38159,
        "s*": 0.10856,
        "p*": 0.61841,
        "d*": 0.0
    },
    "K": {
        "3s": -1.2927265379807145,
        "4s": -0.08921319954956218,
        "3p": -0.692304279607895,
        "p*": 0.307695720392105,
        "d*": 0.0
    },
    "Ca": {
        "3s": -1.7208221279923281,
        "4s": -0.14199514988848455,
        "3p": -1.0284509360911331,
        "p*": -0.02845093609113314,
        "d*": 0.0
    },
    "Sc": {
        "3s": -2.0103937043007374,
        "4s": -0.1577291501305132,
        "3p": -1.2335989631558661,
        "4p": -0.05586868590143592,
        "3d": -0.12587533744234503,
        "d*": 0.874124662557655
    },
    "Ti": {
        "3s": -2.2879,
        "4s": -0.1688,
        "3p": -1.42556,
        "4p": -0.05643,
        "3d": -0.16402,
        "d*": 0.83598
    },
    "V": {
        "3s": -2.5657934011167,
        "4s": -0.17807,
        "3p": -1.6155332390661175,
        "4p": -0.05609,
        "3d": -0.19775,
        "d*": 0.80225,
        "s*": 0.82193,
        "p*": 0.94391
    },
    "Cr": {
        "4s": -0.15401,
        "4p": -0.03965,
        "3d": -0.11496,
        "s*": 0.84599,
        "p*": 0.96035,
        "d*": 0.88504
    },
    "Mn": {
        "3s": -3.1379007342774865,
        "4s": -0.1941,
        "3p": -2.002579132990032,
        "4p": -0.05404,
        "3d": -0.25757,
        "d*": 0.74243,
        "s*": 0.8059,
        "p*": 0.94596
    },
    "Fe": {
        "4s": -0.20142,
        "4p": -0.0526,
        "3d": -0.2849,
        "s*": 0.79858,
        "p*": 0.9474,
        "d*": 0.7151
    },
    "Co": {
        "4s": -0.20846,
        "4p": -0.05096,
        "3d": -0.31095,
        "s*": 0.79154,
        "p*": 0.94904,
        "d*": 0.68905
    },
    "Ni": {
        "4s": -0.21528,
        "3p": -2.618967397930805,
        "4p": -0.04917,
        "3d": -0.33592,
        "s*": 0.78472,
        "d*": 0.66408,
        "p*": 0.95083
    },
    "Cu": {
        "4s": -0.17849,
        "4p": -0.02876,
        "3d": -0.19567,
        "s*": 0.82151,
        "p*": 0.97124,
        "d*": 0.80433
    },
    "Zn": {
        "4s": -0.22849401119463367,
        "4p": -0.04521693396111297,
        "3d": -0.38313993375375827,
        "s*": 0.7715059888053664,
        "p*": 0.9547830660388871,
        "d*": 0.6168600662462418
    },
    "Ga": {
        "4s": -0.33687,
        "4p": -0.10046,
        "s*": 0.66313,
        "p*": 0.89954,
        "d*": 0.0
    },
    "Ge": {
        "4s": -0.43878,
        "4p": -0.14873,
        "s*": 0.56122,
        "p*": 0.85127,
        "d*": 0.0
    },
    "As": {
        "4s": -0.5399,
        "4p": -0.19632,
        "s*": 0.4601,
        "p*": 0.80368,
        "d*": 0.0
    },
    "Se": {
        "4s": -0.64209,
        "4p": -0.2446,
        "s*": 0.35791,
        "p*": 0.7554,
        "d*": 0.0
    },
    "Br": {
        "4s": -0.74622,
        "4p": -0.29411,
        "s*": 0.25378,
        "p*": 0.70589,
        "d*": 0.0
    },
    "Kr": {
        "4s": -0.8528,
        "4p": -0.34511,
        "s*": 0.1472,
        "p*": 0.65489,
        "d*": 0.0
    },
    "Rb": {
        "4s": -1.17435,
        "5s": -0.08682,
        "4p": -0.58927,
        "p*": 0.41073,
        "d*": 0.0
    },
    "Sr": {
        "4s": -1.5030846690240067,
        "5s": -0.13378794616296494,
        "4p": -0.8402212678753389,
        "p*": 0.1597787321246611,
        "d*": 0.0
    },
    "Y": {
        "4s": -1.7630724990518696,
        "5s": -0.1553714285011136,
        "4p": -1.0271923571677584,
        "5p": -0.05498029050646998,
        "4d": -0.09719453273991265,
        "d*": 0.9028054672600874
    },
    "Zr": {
        "4s": -2.0025062891024175,
        "5s": -0.1685799386379645,
        "4p": -1.1947311335466901,
        "5p": -0.05687725747686591,
        "4d": -0.13729600021962401,
        "d*": 0.862703999780376
    },
    "Nb": {
        "5s": -0.15407537007534938,
        "5p": -0.044880930120304106,
        "4d": -0.11789001317296084,
        "s*": 0.84592,
        "p*": 0.95512,
        "d*": 0.8821099868270391,
        "4s": -2.1472562060166465,
        "4p": -1.2721133690950814
    },
    "Mo": {
        "5s": -0.15873221755899894,
        "5p": -0.04268405692848645,
        "4d": -0.14454548227847008,
        "s*": 0.84127,
        "p*": 0.95732,
        "d*": 0.8554545177215299,
        "4s": -2.366079747111923,
        "4p": -1.4183430049432446
    },
    "Ru": {
        "4s": -2.807098271927909,
        "5s": -0.16564,
        "4p": -1.710170999265375,
        "5p": -0.03796,
        "4d": -0.19851,
        "d*": 0.80149,
        "s*": 0.83436,
        "p*": 0.96204
    },
    "Rh": {
        "5s": -0.16838,
        "4p": -1.8573135022692624,
        "5p": -0.03559,
        "4d": -0.22591,
        "s*": 0.83162,
        "d*": 0.77409,
        "p*": 0.96441
    },
    "Pd": {
        "5s": -0.13244,
        "4p": -1.887839049503339,
        "5p": -0.01309,
        "4d": -0.1583,
        "s*": 0.86756,
        "d*": 0.8417,
        "p*": 0.98691
    },
    "Ag": {
        "5s": -0.17303,
        "4p": -2.156373679260095,
        "5p": -0.03101,
        "4d": -0.28165,
        "s*": 0.82697,
        "d*": 0.71835,
        "p*": 0.96899
    },
    "Cd": {
        "5s": -0.21867406491868266,
        "5p": -0.048805092934461315,
        "4d": -0.4374622495538516,
        "s*": 0.7813259350813173,
        "p*": 0.9511949070655387,
        "d*": 0.5625377504461484
    },
    "In": {
        "5s": -0.3114063585491757,
        "5p": -0.0992175362542658,
        "4d": -0.6880506545190608,
        "s*": 0.6885936414508242,
        "p*": 0.9007824637457342,
        "d*": 0.3119493454809392
    },
    "Sn": {
        "5s": -0.39684298199969636,
        "5p": -0.1422001351259059,
        "4d": -0.952162957639688,
        "s*": 0.6031570180003036,
        "p*": 0.8577998648740941,
        "d*": 0.047837042360312054
    },
    "Sb": {
        "5s": -0.48028517036103063,
        "5p": -0.1835440374001944,
        "4d": -1.2334711988758018,
        "s*": 0.5197148296389693,
        "p*": 0.8164559625998056,
        "d*": -0.2334711988758018
    },
    "Te": {
        "5s": -0.5636190106904606,
        "5p": -0.2246500588677877,
        "4d": -1.53272,
        "s*": 0.4363809893095394,
        "p*": 0.7753499411322123,
        "d*": 0.0
    },
    "I": {
        "5s": -0.64774,
        "5p": -0.2661,
        "s*": 0.35226,
        "p*": 0.7339,
        "d*": 0.0
    },
    "Xe": {
        "5s": -0.7331775180500425,
        "5p": -0.30818345193935837,
        "s*": 0.2668224819499575,
        "p*": 0.6918165480606416,
        "d*": 0.0
    },
    "Cs": {
        "5s": -0.98751,
        "6s": -0.08172,
        "5p": -0.50018,
        "p*": 0.49982,
        "d*": 0.0
    },
    "Ba": {
        "5s": -1.24117,
        "6s": -0.12296,
        "5p": -0.69135,
        "p*": 0.0,
        "d*": 0.0
    },
    "Hf": {
        "5s": -2.4617483992657,
        "6s": -0.19292187390666052,
        "5p": -1.3134254723066463,
        "6p": -0.0540093503020234,
        "5d": -0.10584601127081078,
        "d*": 0.8941539887291892
    },
    "Ta": {
        "6s": -0.20574140770526533,
        "6p": -0.054491810202995275,
        "5d": -0.1394436845498639,
        "s*": 0.79426,
        "p*": 0.94551,
        "d*": 0.8605563154501361,
        "5s": -2.6819424390467193,
        "5p": -1.4549050784766915
    },
    "W": {
        "5s": -2.9051297442526822,
        "6s": -0.21632,
        "5p": -1.5955289357259512,
        "6p": -0.05398,
        "5d": -0.17247,
        "d*": 0.82753,
        "s*": 0.78368,
        "p*": 0.94602
    },
    "Re": {
        "6s": -0.22546253256930515,
        "5p": -1.7362500794397973,
        "6p": -0.05290068917872333,
        "5d": -0.20530037955659144,
        "s*": 0.7745374674306948,
        "d*": 0.7946996204434086
    },
    "Os": {
        "6s": -0.2336136042534115,
        "6p": -0.05146239128097571,
        "5d": -0.2381358515186364,
        "s*": 0.7663863957465885,
        "p*": 0.94854,
        "d*": 0.7618641484813636,
        "5p": -1.877612031354627
    },
    "Ir": {
        "6s": -0.2410511019320077,
        "6p": -0.04978409378565279,
        "5d": -0.2710835995371712,
        "s*": 0.7589488980679923,
        "p*": 0.95022,
        "d*": 0.7289164004628288,
        "5p": -2.019975616278318
    },
    "Pt": {
        "6s": -0.21793139470079848,
        "6p": -0.03535274276506047,
        "5d": -0.23471315184256072,
        "s*": 0.7820686052992015,
        "p*": 0.96465,
        "d*": 0.7652868481574393,
        "5p": -2.0696648258804933
    },
    "Au": {
        "6s": -0.22226,
        "6p": -0.03263,
        "5d": -0.26199,
        "s*": 0.77774,
        "p*": 0.96737,
        "d*": 0.73801
    },
    "Hg": {
        "6s": -0.2606860779688589,
        "5p": -2.4554765747287997,
        "6p": -0.043916474665032505,
        "5d": -0.3711472638348927,
        "s*": 0.7393139220311411,
        "d*": 0.6288527361651073
    },
    "Tl": {
        "6s": -0.35928085642667945,
        "6p": -0.09497093692677082,
        "5d": -0.575284245638937,
        "s*": 0.6407191435733206,
        "p*": 0.9050290630732292,
        "d*": 0.424715754361063
    },
    "Pb": {
        "6s": -0.45015581291840656,
        "6p": -0.13645250125393177,
        "5d": -0.7837211185509239,
        "s*": 0.5498441870815934,
        "p*": 0.8635474987460683,
        "d*": 0.21627888144907614
    },
    "Bi": {
        "6s": -0.5389410094782598,
        "6p": -0.1755604665891423,
        "5d": -1.0004914916154541,
        "s*": 0.46105899052174015,
        "p*": 0.8244395334108577,
        "d*": -0.0004914916154541427
    },
    "Rn": {
        "6s": -0.8084185412165587,
        "6p": -0.2903772104091421,
        "s*": 0.19158145878344135,
        "p*": 0.7096227895908579,
        "d*": 0.0
    }
}