# spin-orbit coupling (SOC) strength database, in the unit of Hartree
# This is A dictionary of dictionaries. 
# The first dictionary is the element name, and the second dictionary is the orbital name. 
# The orbital name is the key, and the value is the SOC  strength.
# For now, all the values are 0, we will fill it when we use 0 as initial as train the model to get the values
# gradually we can get all the values as a database.


# Follows the onsite energy database, we consider the atom types list here.
# Contains the elements as follows:

#    AtomSymbol=[
#     'H',                                                                                                  'He', 
#     'Li', 'Be',                                                             'B',  'C',  'N',  'O',  'F',  'Ne', 
#     'Na', 'Mg',                                                             'Al', 'Si', 'P',  'S',  'Cl', 'Ar',
#     'K',  'Ca', 'Sc', 'Ti', 'V',  'Cr', 'Mn', 'Fe', 'Co', 'Ni', 'Cu', 'Zn', 'Ga', 'Ge', 'As', 'Se', 'Br', 'Kr', 
#     'Rb', 'Sr', 'Y',  'Zr', 'Nb', 'Mo',     , 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', 'In', 'Sn', 'Sb', 'Te', 'I',  'Xe', 
#     'Cs', 'Ba',       'Hf', 'Ta', 'W',  'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', 'Tl', 'Pb', 'Bi',             'Rn'
#     ]



soc_strength_database = \
{
    "H": {
        "1s": 0,
        "s*": 0,
        "p*": 0
    },
    "He": {
        "1s": 0,
        "s*": 0,
        "p*": 0
    },
    "Li": {
        "2s": 0,
        "2p": 0,
        "s*": 0
    },
    "Be": {
        "2s": 0,
        "2p": 0,
        "s*": 0
    },
    "B": {
        "2s": 0,
        "2p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "C": {
        "2s": 0,
        "2p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "N": {
        "2s": 0,
        "2p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "O": {
        "2s": 0,
        "2p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "F": {
        "2s": 0,
        "2p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Ne": {
        "2s": 0,
        "2p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Na": {
        "3s": 0,
        "3p": 0,
        "s*": 0,
        "2p": 0,
        "d*": 0
    },
    "Mg": {
        "2s": 0,
        "3s": 0,
        "2p": 0,
        "3p": 0,
        "d*": 0,
        "s*": 0
    },
    "Al": {
        "3s": 0,
        "3p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Si": {
        "3s": 0,
        "3p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "P": {
        "3s": 0,
        "3p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "S": {
        "3s": 0,
        "3p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Cl": {
        "3s": 0,
        "3p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Ar": {
        "3s": 0,
        "3p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "K": {
        "3s": 0,
        "4s": 0,
        "3p": 0,
        "p*": 0,
        "d*": 0
    },
    "Ca": {
        "3s": 0,
        "4s": 0,
        "3p": 0,
        "p*": 0,
        "d*": 0
    },
    "Sc": {
        "3s": 0,
        "4s": 0,
        "3p": 0,
        "4p": 0,
        "3d": 0,
        "d*": 0
    },
    "Ti": {
        "3s": 0,
        "4s": 0,
        "3p": 0,
        "4p": 0,
        "3d": 0,
        "d*": 0
    },
    "V": {
        "3s": 0,
        "4s": 0,
        "3p": 0,
        "4p": 0,
        "3d": 0,
        "d*": 0,
        "s*": 0,
        "p*": 0
    },
    "Cr": {
        "4s": 0,
        "4p": 0,
        "3d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Mn": {
        "3s": 0,
        "4s": 0,
        "3p": 0,
        "4p": 0,
        "3d": 0,
        "d*": 0,
        "s*": 0,
        "p*": 0
    },
    "Fe": {
        "4s": 0,
        "4p": 0,
        "3d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Co": {
        "4s": 0,
        "4p": 0,
        "3d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Ni": {
        "4s": 0,
        "3p": 0,
        "4p": 0,
        "3d": 0,
        "s*": 0,
        "d*": 0,
        "p*": 0
    },
    "Cu": {
        "4s": 0,
        "4p": 0,
        "3d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Zn": {
        "4s": 0,
        "4p": 0,
        "3d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Ga": {
        "4s": 0,
        "4p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Ge": {
        "4s": 0,
        "4p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "As": {
        "4s": 0,
        "4p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Se": {
        "4s": 0,
        "4p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Br": {
        "4s": 0,
        "4p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Kr": {
        "4s": 0,
        "4p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Rb": {
        "4s": 0,
        "5s": 0,
        "4p": 0,
        "p*": 0,
        "d*": 0
    },
    "Sr": {
        "4s": 0,
        "5s": 0,
        "4p": 0,
        "p*": 0,
        "d*": 0
    },
    "Y": {
        "4s": 0,
        "5s": 0,
        "4p": 0,
        "5p": 0,
        "4d": 0,
        "d*": 0
    },
    "Zr": {
        "4s": 0,
        "5s": 0,
        "4p": 0,
        "5p": 0,
        "4d": 0,
        "d*": 0
    },
    "Nb": {
        "5s": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0,
        "4s": 0,
        "4p": 0
    },
    "Mo": {
        "5s": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0,
        "4s": 0,
        "4p": 0
    },
    "Ru": {
        "4s": 0,
        "5s": 0,
        "4p": 0,
        "5p": 0,
        "4d": 0,
        "d*": 0,
        "s*": 0,
        "p*": 0
    },
    "Rh": {
        "5s": 0,
        "4p": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "d*": 0,
        "p*": 0
    },
    "Pd": {
        "5s": 0,
        "4p": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "d*": 0,
        "p*": 0
    },
    "Ag": {
        "5s": 0,
        "4p": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "d*": 0,
        "p*": 0
    },
    "Cd": {
        "5s": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "In": {
        "5s": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Sn": {
        "5s": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Sb": {
        "5s": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Te": {
        "5s": 0,
        "5p": 0,
        "4d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "I": {
        "5s": 0,
        "5p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Xe": {
        "5s": 0,
        "5p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Cs": {
        "5s": 0,
        "6s": 0,
        "5p": 0,
        "p*": 0,
        "d*": 0
    },
    "Ba": {
        "5s": 0,
        "6s": 0,
        "5p": 0,
        "p*": 0,
        "d*": 0
    },
    "Hf": {
        "5s": 0,
        "6s": 0,
        "5p": 0,
        "6p": 0,
        "5d": 0,
        "d*": 0
    },
    "Ta": {
        "6s": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0,
        "5s": 0,
        "5p": 0
    },
    "W": {
        "5s": 0,
        "6s": 0,
        "5p": 0,
        "6p": 0,
        "5d": 0,
        "d*": 0,
        "s*": 0,
        "p*": 0
    },
    "Re": {
        "6s": 0,
        "5p": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "d*": 0
    },
    "Os": {
        "6s": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0,
        "5p": 0
    },
    "Ir": {
        "6s": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0,
        "5p": 0
    },
    "Pt": {
        "6s": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0,
        "5p": 0
    },
    "Au": {
        "6s": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Hg": {
        "6s": 0,
        "5p": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "d*": 0
    },
    "Tl": {
        "6s": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Pb": {
        "6s": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Bi": {
        "6s": 0,
        "6p": 0,
        "5d": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    },
    "Rn": {
        "6s": 0,
        "6p": 0,
        "s*": 0,
        "p*": 0,
        "d*": 0
    }
}